from typing import Dict, List

from sparkdesk.authentication import Authentication
from sparkdesk.chat_utils import limit_chat_history
from sparkdesk.completion import ChatCompletion


class Chat:
    """
    Class for creating chat
    """

    def __init__(
        self,
        authentication: Authentication,
        app_id: str,
        user_id="default_user",
        domain="general",
        temperature=0.5,
        max_tokens=2048,
    ):
        """
        Constructor for the class.

        Args:
            authentication (Authentication):
                The authentication object used to authenticate the API calls.
            app_id (str):
                The ID of the application.
            user_id (str, optional):
                The ID of the user. Defaults to "default_user".
            domain (str, optional):
                The domain of the user. Defaults to "general".
            temperature (float, optional):
                The temperature to control the randomness of the generated output.
                Defaults to 0.5.
            max_tokens (int, optional):
                The maximum number of tokens to generate. Defaults to 2048.
        """
        self.chat_completion = ChatCompletion(
            authentication, app_id, user_id, domain, temperature, max_tokens
        )
        self.chat_history: List[Dict[str, str]] = []

    def reset_chat_context(self):
        self.chat_history = []

    def ask(self, question: str):
        """
        Appends the user's question to the chat context,
        generates responses using the chat_completion model,
        and adds the assistant's response to the chat context.

        Args:
            question (str): The user's question.

        Yields:
            dict: A dictionary representing each response generated by
                the chat_completion model.
            reference: https://www.xfyun.cn/doc/spark/Web.html
        """

        self.chat_history.append({"role": "user", "content": question})
        self.chat_history = limit_chat_history(self.chat_history)

        response_messages = []
        responses = self.chat_completion.create(self.chat_history)
        try:
            for response in responses:
                response_messages.extend(
                    [e["content"] for e in response["payload"]["choices"]["text"]]
                )
                yield response
        finally:  # GeneratorExit
            self.chat_history.append(
                {"role": "assistant", "content": "".join(response_messages)}
            )



import click

from sparkdesk.authentication import Authentication
from sparkdesk.chat import Chat


@click.command()
@click.option("--app-id", required=True)
@click.option("--api-secret", required=True)
@click.option("--api-key", required=True)
def _chat(app_id: str, api_secret: str, api_key: str):
    authentication = Authentication(
        api_key=api_key, api_secret=api_secret, api_version="v1.1"
    )
    chat = Chat(authentication, app_id)
    print("-----------------------")
    print("q: 退出会话")
    print("reset: 清除会话上下文")
    print("-----------------------")
    while True:
        question = input("Ask: ")
        if question in {"q", "quit", "exit", "stop"}:
            break
        if question == "reset":
            chat.reset_chat_context()
        responses = chat.ask(question)
        for response in responses:
            for content in [
                e["content"] for e in response["payload"]["choices"]["text"]
            ]:
                print(content, end="")
        print("\n")


if __name__ == "__main__":
    _chat()



import base64
import hmac
from datetime import datetime, timezone
from urllib.parse import urlencode, urlparse


class Authentication:
    def __init__(self, api_key: str, api_secret: str, api_version="v2.1"):
        self.api_key = api_key
        self.api_secret = api_secret
        self.api_version = api_version

    @property
    def _api_url(self):
        return f"wss://spark-api.xf-yun.com/{self.api_version}/chat"

    def get_authorization_url(self):
        parsed_api_url = urlparse(self._api_url)
        date = datetime.now(timezone.utc).strftime("%a, %d %b %Y %H:%M:%S %Z")

        signature_origin = (
            f"host: {parsed_api_url.netloc}\n"
            f"date: {date}\n"
            f"GET {parsed_api_url.path} HTTP/1.1"
        )
        signature = base64.b64encode(
            hmac.new(
                self.api_secret.encode(), signature_origin.encode(), digestmod="sha256"
            ).digest()
        ).decode()
        authorization_origin = (
            f'api_key="{self.api_key}", algorithm="hmac-sha256", '
            f'headers="host date request-line", signature="{signature}"'
        )
        authorization = base64.b64encode(authorization_origin.encode()).decode()
        params = {
            "authorization": authorization,
            "date": date,
            "host": parsed_api_url.netloc,
        }

        return f"{self._api_url}?{urlencode(params)}"



import json
import logging
from typing import Dict, List

from websockets.sync.client import connect

from sparkdesk.authentication import Authentication

logger = logging.getLogger(__name__)


class BaseChatCompletion:
    """
    Class for creating chat completion.
    """

    def __init__(
        self,
        authentication: Authentication,
        app_id: str,
        user_id: str = "default_user",
        domain: str = "general",
        temperature: float = 0.5,
        max_tokens: int = 2048,
    ):
        """
        Constructor for the class.

        Args:
            authentication (Authentication):
                The authentication object used to authenticate the API calls.
            app_id (str):
                The ID of the application.
            user_id (str, optional):
                The ID of the user. Defaults to "default_user".
            domain (str, optional):
                The domain of the user. Defaults to "general".
            temperature (float, optional):
                The temperature to control the randomness of the generated output.
                Defaults to 0.5.
            max_tokens (int, optional):
                The maximum number of tokens to generate. Defaults to 2048.
        """
        self.authentication = authentication
        self.app_id = app_id
        self.user_id = user_id
        self.domain = domain
        self.temperature = temperature
        self.max_tokens = max_tokens

    def _build_ws_input_message(self, message_text: List[Dict[str, str]]):
        return json.dumps(
            {
                "header": {
                    "app_id": self.app_id,
                    "uid": self.user_id,
                },
                "parameter": {
                    "chat": {
                        "domain": self.domain,
                        "temperature": self.temperature,
                        "max_tokens": self.max_tokens,
                    }
                },
                "payload": {"message": {"text": message_text}},
            }
        )

    @staticmethod
    def _process_ws_response_message(response_message: str):
        return json.loads(response_message)


class ChatCompletion(BaseChatCompletion):
    def create(self, message_text: List[Dict[str, str]]):
        """
        Create chat completion.
        reference: https://www.xfyun.cn/doc/spark/Web.html

        Args:
            message_text:
                A list of dictionaries containing the message content.
                The total length of the message content should be less than 8192.
                one token is approximately equal to 1.5 Chinese characters or
                0.8 English words.
                ex: [{"role": "user", "content": "Hi"}]

        Yields:
            An iterator that yields the processed response messages from the websocket.
        """
        with connect(self.authentication.get_authorization_url()) as websocket:
            input_message = self._build_ws_input_message(message_text)
            logger.debug(f"send: {input_message}")
            websocket.send(input_message)

            response_message = websocket.recv()
            logger.debug(f"recv: {response_message}")
            while True:
                processed_ws_response_message = self._process_ws_response_message(
                    response_message
                )
                yield processed_ws_response_message
                # if last message, exit
                if processed_ws_response_message["header"]["status"] == 2:
                    break
                response_message = websocket.recv()
                logger.debug(f"recv: {response_message}")



from typing import Dict, List

from sparkdesk.consts import MAX_REQUEST_TOKENS_COUNT


def count_tokens(text: str) -> int:
    """
    Count the number of tokens in a text.

    A token is approximately equal to 1.5 Chinese characters or 0.8 English words.

    Args:
        text (str): The input text.

    Returns:
        int: The number of tokens in the text.
    """
    return len(text.encode("utf-8", errors="ignore")) // 2


def limit_chat_history(
    chat_history: List[Dict[str, str]],
    max_request_tokens_count: int = MAX_REQUEST_TOKENS_COUNT,
) -> List[Dict[str, str]]:
    """
    Limits the chat history by removing messages that exceed
    the maximum number of request tokens count.
    """
    total_count = 0
    new_chat_history: List[Dict[str, str]] = []
    for message in reversed(chat_history):
        content = message["content"]
        total_count += count_tokens(content)
        if total_count > MAX_REQUEST_TOKENS_COUNT:
            remain_tokens = total_count - max_request_tokens_count
            message["content"] = (content.encode("utf-8")[:remain_tokens]).decode(
                "utf-8", errors="ignore"
            )
            new_chat_history.insert(0, message)
            break
        new_chat_history.insert(0, message)
    return new_chat_history
